"""Thin wrapper around the UK Environment Agent Real-time Flood Monitoring API."""
import enum
from typing import Any, Dict, List

import aiohttp


class Status(enum.Enum):

    """The status of the monitoring station."""

    ACTIVE = "Active"
    CLOSED = "Closed"
    SUSPENDED = "Suspended"


async def get_stations(
    session: aiohttp.ClientSession,
    parameter_name: str = None,
    parameter: str = None,
    qualifier: str = None,
    label: str = None,
    town: str = None,
    river_name: str = None,
    station: str = None,
    status: Status = Status.ACTIVE,
) -> List[Dict[str, Any]]:
    """Returns a list of stations."""
    params = {}

    if parameter_name:
        params["parameterName"] = parameter_name
    if parameter:
        params["parameter"] = parameter
    if qualifier:
        params["qualifier"] = qualifier
    if label:
        params["label"] = label
    if town:
        params["town"] = town
    if river_name:
        params["riverName"] = river_name
    if station:
        params["stationReference"] = station
    if status:
        params["status"] = status.value

    response = await session.get(
        "http://environment.data.gov.uk/flood-monitoring/id/stations",
        raise_for_status=True,
        timeout=aiohttp.ClientTimeout(total=30),
        params=params,
    )
    results = await response.json()

    return results["items"]


async def get_station(session: aiohttp.ClientSession, station: str) -> Dict[str, Any]:
    """Returns all data for a given station."""
    response = await session.get(
        f"http://environment.data.gov.uk/flood-monitoring/id/stations/{station}",
        raise_for_status=True,
        timeout=aiohttp.ClientTimeout(total=15),
    )
    results = await response.json()

    return results["items"]



import pytest


def pytest_addoption(parser):
    parser.addoption(
        "--integration",
        action="store_true",
        help="Run integration tests.",
    )


def pytest_configure(config):
    config.addinivalue_line("markers", "integration: mark test as an integration test")


def pytest_runtest_setup(item):
    isintegration = len(list(item.iter_markers(name="integration"))) > 0
    if isintegration:
        if not item.config.getoption("--integration"):
            pytest.skip("integration tests not selected in this run")



import asyncio
import json
import pathlib
from unittest import mock

import aiohttp
import pytest

import aioeafm


async def mock_session(path):
    with open(pathlib.Path(__file__).parent / "fixtures" / path) as fp:
        fixture = json.load(fp)

    session = mock.Mock()
    get = session.get.return_value = asyncio.Future()

    response = mock.Mock()
    get_json = response.json.return_value = asyncio.Future()
    get_json.set_result(fixture)

    get.set_result(response)

    return session


async def test_get_stations():
    session = await mock_session("get_stations.json")
    stations = await aioeafm.get_stations(session)

    station = stations[0]
    assert station["label"] == "Bourton Dickler"

    measures = station["measures"]
    assert measures[0]["parameter"] == "level"
    assert measures[0]["qualifier"] == "Downstream Stage"


async def test_get_station():
    session = await mock_session("get_station_1491TH.json")
    station = await aioeafm.get_station(session, "1491TH")

    assert station["label"] == "Kings Mill"

    measures = station["measures"]
    assert measures[0]["parameter"] == "level"
    assert measures[0]["qualifier"] == "Downstream Stage"
    assert measures[0]["latestReading"]["value"] == 1.877


@pytest.mark.integration
async def test_integration_get_stations():
    async with aiohttp.ClientSession() as session:
        stations = await aioeafm.get_stations(session)
        assert isinstance(stations, list)


@pytest.mark.integration
async def test_integration_get_station():
    async with aiohttp.ClientSession() as session:
        station = await aioeafm.get_station(session, "1771TH")
        assert isinstance(station, dict)
